@@ -1,8 +1,8 @@
 use super::*;
 
-/// Main entry point into just binary.
+/// Main entry point into `just`. Parse arguments from `args` and run.
 #[allow(clippy::missing_errors_doc)]
-pub fn run() -> Result<(), i32> {
+pub fn run(args: impl Iterator<Item = impl Into<OsString> + Clone>) -> Result<(), i32> {
   #[cfg(windows)]
   ansi_term::enable_ansi_support().ok();
 
@@ -11,12 +11,16 @@ pub fn run() -> Result<(), i32> {
       .filter("JUST_LOG")
       .write_style("JUST_LOG_STYLE"),
   )
-  .init();
+  .try_init()
+  .ok();
 
   let app = Config::app();
 
   info!("Parsing command line arguments…");
-  let matches = app.get_matches();
+  let matches = app.try_get_matches_from(args).map_err(|err| {
+    err.print().ok();
+    err.exit_code()
+  })?;
 
   let config = Config::from_matches(&matches).map_err(Error::from);
 
