@@ -4,19 +4,22 @@ use super::*;
 pub(crate) enum Setting<'src> {
   AllowDuplicateRecipes(bool),
   AllowDuplicateVariables(bool),
-  DotenvFilename(String),
+  DotenvFilename(StringLiteral<'src>),
   DotenvLoad(bool),
-  DotenvPath(String),
+  DotenvPath(StringLiteral<'src>),
   DotenvRequired(bool),
   Export(bool),
   Fallback(bool),
   IgnoreComments(bool),
   PositionalArguments(bool),
   Quiet(bool),
-  Shell(Shell<'src>),
-  Tempdir(String),
+  ScriptInterpreter(Interpreter<'src>),
+  Shell(Interpreter<'src>),
+  Tempdir(StringLiteral<'src>),
+  Unstable(bool),
   WindowsPowerShell(bool),
-  WindowsShell(Shell<'src>),
+  WindowsShell(Interpreter<'src>),
+  WorkingDirectory(StringLiteral<'src>),
 }
 
 impl<'src> Display for Setting<'src> {
@@ -31,10 +34,16 @@ impl<'src> Display for Setting<'src> {
       | Self::IgnoreComments(value)
       | Self::PositionalArguments(value)
       | Self::Quiet(value)
+      | Self::Unstable(value)
       | Self::WindowsPowerShell(value) => write!(f, "{value}"),
-      Self::Shell(shell) | Self::WindowsShell(shell) => write!(f, "{shell}"),
-      Self::DotenvFilename(value) | Self::DotenvPath(value) | Self::Tempdir(value) => {
-        write!(f, "{value:?}")
+      Self::ScriptInterpreter(shell) | Self::Shell(shell) | Self::WindowsShell(shell) => {
+        write!(f, "[{shell}]")
+      }
+      Self::DotenvFilename(value)
+      | Self::DotenvPath(value)
+      | Self::Tempdir(value)
+      | Self::WorkingDirectory(value) => {
+        write!(f, "{value}")
       }
     }
   }
